import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';

import '../form.dart';

typedef FastSwitchTitleBuilder = Widget Function(FastSwitchState field);

@immutable
class FastSwitch extends FastFormField<bool> {
  const FastSwitch({
    bool? adaptive,
    bool autofocus = false,
    AutovalidateMode autovalidateMode = AutovalidateMode.onUserInteraction,
    FormFieldBuilder<bool>? builder,
    EdgeInsetsGeometry? contentPadding,
    InputDecoration? decoration,
    bool enabled = true,
    String? helperText,
    bool initialValue = false,
    Key? key,
    String? labelText,
    required String name,
    ValueChanged<bool>? onChanged,
    VoidCallback? onReset,
    FormFieldSetter<bool>? onSaved,
    FormFieldValidator<bool>? validator,
    this.activeColor,
    this.activeThumbImage,
    this.activeTrackColor,
    this.controlAffinity = ListTileControlAffinity.platform,
    this.dragStartBehavior = DragStartBehavior.start,
    this.dense,
    this.enableFeedback,
    this.errorBuilder,
    this.focusNode,
    this.helperBuilder,
    this.hoverColor,
    this.inactiveThumbColor,
    this.inactiveThumbImage,
    this.inactiveTrackColor,
    this.isThreeLine = false,
    this.secondary,
    this.selectedTileColor,
    this.shapeBorder,
    this.subtitle,
    this.thumbColor,
    this.tileColor,
    this.titleText,
    this.titleBuilder,
    this.visualDensity,
  }) : super(
          adaptive: adaptive,
          autofocus: autofocus,
          autovalidateMode: autovalidateMode,
          builder: builder ?? switchBuilder,
          contentPadding: contentPadding,
          decoration: decoration,
          enabled: enabled,
          helperText: helperText,
          initialValue: initialValue,
          key: key,
          labelText: labelText,
          name: name,
          onChanged: onChanged,
          onReset: onReset,
          onSaved: onSaved,
          validator: validator,
        );

  final Color? activeColor;
  final ImageProvider<Object>? activeThumbImage;
  final Color? activeTrackColor;
  final ListTileControlAffinity controlAffinity;
  final bool? dense;
  final DragStartBehavior dragStartBehavior;
  final bool? enableFeedback;
  final FastErrorBuilder<bool>? errorBuilder;
  final FocusNode? focusNode;
  final FastHelperBuilder<bool>? helperBuilder;
  final Color? hoverColor;
  final Color? inactiveThumbColor;
  final ImageProvider<Object>? inactiveThumbImage;
  final Color? inactiveTrackColor;
  final bool isThreeLine;
  final Widget? secondary;
  final Color? selectedTileColor;
  final ShapeBorder? shapeBorder;
  final Widget? subtitle;
  final Color? thumbColor;
  final Color? tileColor;
  final String? titleText;
  final FastSwitchTitleBuilder? titleBuilder;
  final VisualDensity? visualDensity;

  @override
  FastSwitchState createState() => FastSwitchState();
}

class FastSwitchState extends FastFormFieldState<bool> {
  @override
  FastSwitch get widget => super.widget as FastSwitch;
}

Text switchTitleBuilder(FastSwitchState field) {
  return Text(
    field.widget.titleText!,
    style: TextStyle(
      fontSize: 14.0,
      color: field.value! ? Colors.black : Colors.grey,
    ),
  );
}

InputDecorator materialSwitchBuilder(FormFieldState<bool> field) {
  final widget = (field as FastSwitchState).widget;
  final titleBuilder = widget.titleBuilder ?? switchTitleBuilder;

  return InputDecorator(
    decoration: field.decoration,
    child: SwitchListTile.adaptive(
      activeColor: widget.activeColor,
      activeThumbImage: widget.activeThumbImage,
      activeTrackColor: widget.activeTrackColor,
      autofocus: widget.autofocus,
      contentPadding: widget.contentPadding,
      controlAffinity: widget.controlAffinity,
      dense: widget.dense,
      enableFeedback: widget.enableFeedback,
      focusNode: widget.focusNode,
      hoverColor: widget.hoverColor,
      inactiveThumbColor: widget.inactiveThumbColor,
      inactiveThumbImage: widget.inactiveThumbImage,
      inactiveTrackColor: widget.inactiveTrackColor,
      isThreeLine: widget.isThreeLine,
      onChanged: widget.enabled ? field.didChange : null,
      secondary: widget.secondary,
      selected: field.value!,
      selectedTileColor: widget.selectedTileColor,
      shape: widget.shapeBorder,
      subtitle: widget.subtitle,
      tileColor: widget.tileColor,
      title: widget.titleText is String ? titleBuilder(field) : null,
      value: field.value!,
      visualDensity: widget.visualDensity,
    ),
  );
}

CupertinoFormRow cupertinoSwitchBuilder(FormFieldState<bool> field) {
  final widget = (field as FastSwitchState).widget;

  return CupertinoFormRow(
    padding: widget.contentPadding,
    prefix: widget.labelText is String ? Text(widget.labelText!) : null,
    helper: (widget.helperBuilder ?? helperBuilder)(field),
    error: (widget.errorBuilder ?? errorBuilder)(field),
    child: CupertinoSwitch(
      activeColor: widget.activeColor,
      dragStartBehavior: widget.dragStartBehavior,
      onChanged: widget.enabled ? field.didChange : null,
      thumbColor: widget.thumbColor,
      trackColor: widget.activeTrackColor,
      value: field.value!,
    ),
  );
}

Widget switchBuilder(FormFieldState<bool> field) {
  FormFieldBuilder<bool> builder = materialSwitchBuilder;

  if ((field as FastSwitchState).adaptive) {
    final platform = Theme.of(field.context).platform;
    if (platform == TargetPlatform.iOS) builder = cupertinoSwitchBuilder;
  }

  return builder(field);
}
